#include <EEPROM.h>

// eeprom data:
// byte min_power, byte max_power, byte def_power, ushort min_temp, ushort max_temp, ushort def_temp
// (ushort temp, double TFR), ...., (0xFFFF, double 'padding')

#define MIN_POWER 7
#define MAX_POWER 20
#define DEF_POWER 10
#define MIN_TEMP 100
#define MAX_TEMP 300
#define DEF_TEMP 200

/*
  Writes n bytes from src in SRAM to dest in EEPROM
*/
void eeprom_write_n_bytes (byte *dest, byte *src, int n) {
  for(int i = 0; i < n; i++, dest++, src++)
    EEPROM.write(dest, *src);
}

/*
  Reads n bytes from src address in EEPROM to dest address in SRAM
*/
void eeprom_read_n_bytes (byte *dest, byte *src, int n) {
  for(int i = 0; i < n; i++, dest++, src++)
    *dest = EEPROM.read(src);
}

int l = 13;

void setup() {
  Serial.begin(9600);
  /*
    Writing data to EEPROM
  */
  unsigned short temp_list[] = {80,	100, 120,	140, 160,	180, 200,	220, 240,	260, 280,	300, 320};
  double TFR_list[] = {1.006575, 1.008757, 1.010919, 1.013081, 1.015243, 1.017405, 1.019568, 1.021874,
                            1.024216, 1.026559,	1.028901,	1.031243,	1.033670};

  unsigned short min_temp = (unsigned short)MIN_TEMP;
  unsigned short max_temp = (unsigned short)MAX_TEMP;
  unsigned short def_temp = (unsigned short)DEF_TEMP;

  EEPROM.write(0x0, (byte)MIN_POWER);
  EEPROM.write(0x1, (byte)MAX_POWER);
  EEPROM.write(0x2, (byte)DEF_POWER);
  eeprom_write_n_bytes(0x3, (byte *)&min_temp, sizeof(unsigned short));
  eeprom_write_n_bytes(0x3 + sizeof(unsigned short), (byte *)&max_temp, sizeof(unsigned short));
  eeprom_write_n_bytes(0x3 + 2 * sizeof(unsigned short), (byte *)&def_temp, sizeof(unsigned short));

  for(auto i = 0; i < l; i++) {
    eeprom_write_n_bytes(0x3 + 3 * sizeof(unsigned short) + i * (sizeof(unsigned short) + sizeof(double))
                          , (byte *)&temp_list[i], sizeof(unsigned short));
    eeprom_write_n_bytes(0x3 + 3 * sizeof(unsigned short) + i * (sizeof(unsigned short) + sizeof(double))
                          + sizeof(unsigned short), (byte *)&TFR_list[i], sizeof(double));
  }

  unsigned short fill = 0xFFFF;
  eeprom_write_n_bytes(0x3 + 3 * sizeof(unsigned short) + l * (sizeof(unsigned short) + sizeof(double)),
                      (byte *)&fill, sizeof(unsigned short));
}

void loop() {
  /*
    Checking previously written data
  */

  unsigned short short_value;
  double double_value;
  Serial.println(EEPROM.read(0x0));
  Serial.println(EEPROM.read(0x1));
  Serial.println(EEPROM.read(0x2));

  eeprom_read_n_bytes((byte *)&short_value, 0x3, sizeof(unsigned short));
  Serial.println(short_value);
  eeprom_read_n_bytes((byte *)&short_value, 0x3 + sizeof(unsigned short), sizeof(unsigned short));
  Serial.println(short_value);
  eeprom_read_n_bytes((byte *)&short_value, 0x3 + 2 * sizeof(unsigned short), sizeof(unsigned short));
  Serial.println(short_value);

  int i = 0;
  while(short_value != 0xFFFF) {
    eeprom_read_n_bytes((byte *)&short_value, 0x3 + 3 * sizeof(unsigned short) + i * (sizeof(unsigned short) + sizeof(double)),
                        sizeof(unsigned short));
    Serial.println(short_value);
    eeprom_read_n_bytes((byte *)&double_value, 0x3 + 3 * sizeof(unsigned short) + i * (sizeof(unsigned short) + sizeof(double))
                        + sizeof(unsigned short), sizeof(double));
    Serial.println(double_value, "%f.10");
    i++;
  }

  while(true);
}
